#!/bin/bash -e

# -------------------------------------------------------------------- #
# Netcdf and utilities installation script                             #
#                                                                      #
# Author : Mickael ACCENSI - IFREMER                                   #
# License : CC BY-SA                                                   #
# Creation date : 04-05-2020                                           #
# Modication date : 16-04-2021                                         #
# -------------------------------------------------------------------- #


# -------------------------------------------------------------------- #
# Important Note                                                       #
#                                                                      #
# This script has been designed for IFREMER HPC installation based on  #
# existing compiler libraries available via modules. This script must  #
# be adapted for your use depending on your system. Please refer to    #
# section 5 - Initialize compiler environment                          #
#                                                                      #
# For some librairies, temporary modules "cmake and "conda" are loaded #
# It must be adapted to your system. Please refer to JASPER and        #
# ECCODES installation sections                                        #
# -------------------------------------------------------------------- #


# -------------------------------------------------------------------- #
# Installation remarks                                                 #
#                                                                      #
# If zlib library is available on your system, it is highly recommended#
# to use it instead of installing a new one to avoid conflicts with    #
# other programs already installed. On IFREMER HPC, zlib is in /lib64  #
#                                                                      #
# For PGI compiler, nco and cdo tools are not compiled due to bad PGI  #
# installtion on IFREMER HPC                                           #
# -------------------------------------------------------------------- #


# -------------------------------------------------------------------- #
# Frequently encountered errors                                        #
#                                                                      #
# error lib not found or badbly recognized                             #
# -> check lib not found with ldd or file -L                           #
# -> force linking with -L/path/name -lname in all dependencies        #
#                                                                      #
# error with mpi function not found                                    #
# -> define MPI_ROOT                                                   #
#                                                                      #
# error when linking with icpc                                         #
# -> add -DMPI_NO_CPPBIND to MPT CFLAGS and CPPFLAGS                   #
#                                                                      #
# fPIC can be source of error in module reading                        #
# -> try with and without in CFLAGS and/or FFLAGS                      #
#                                                                      #
# module and path in $PATH and $LD_LIBRARY can affect compiler         #
# -> try the minimal config or in static                               #
#                                                                      #
# compiler error is often due to conflicting env variable              #
# -> run as "bash install_netcdf.sh"                                   #
#                                                                      #
# error compiling nco with esmf                                        #
# -> do not use option --enable-esmf, it is not needed for remapping   #
# -------------------------------------------------------------------- #


# -------------------------------------------------------------------- #
# Script architecture                                                  #
#                                                                      #
## Section 1 - input arguments                                         #
## Section 2 - download from website                                   #
## Section 3 - download from ftp                                       #
## Section 4 - extract tar to src                                      #
## Section 5 - initialize compiler environment                         #
## Section 6 - initialize environment variables                        #
## Section 7 - netcdf libraries installation                           #
## Section 8 - nco installation                                        #
## Section 9 - ncview installation                                     #
## Section 10 - cdo installation                                       #
## Section 11 - cdl installation                                       #
## Section 12 - successfull exit message                               #
# -------------------------------------------------------------------- #


# -------------------------------------------------------------------- #
# How to run it                                                        #
#                                                                      #
# bash install_netcdf.bash <COMP>                                      #
# with COMP : PGI | GNU | INTEL | MPT                                  #
# -------------------------------------------------------------------- #


####################################################################################################
## Section 1 - input arguments

if [ $# -ne 1 ]; then
  echo 'need to set the compiler [PGI | GNU | INTEL | MPT]'
  exit 1
else
  compiler=$1
fi

# enable or disable download sources from ifremer ftp
ftp=1
# enable or disable download sources from each provider website
website=0
# extract the sources
extract=1
# install the netcdf libraries
netcdf=1
# install netcdf utilies
nco=1
ncview=1
cdo=1
cdl=0

mkdir -p $compiler
cd $compiler
NETCDF=${PWD}


####################################################################################################
## Section 2 - download from website

if [ ${website} -eq 1 ]; then

 cd $NETCDF
 rm -rf TAR && mkdir -p TAR && cd TAR


# flex libexpat udunits can be downloaded here : 
  wget -mnH --cut-dirs=5 https://github.com/westes/flex/releases/download/v2.6.4/flex-2.6.4.tar.gz
  wget -mnH --cut-dirs=3 https://github.com/libexpat/libexpat/archive/R_2_2_5.tar.gz
  mv R_2_2_5.tar.gz libexpat-2.2.5.tar.gz
  wget -mnH --cut-dirs=2 ftp://ftp.unidata.ucar.edu/pub/udunits/udunits-2.2.26.tar.gz

# hdf5 curl zlib sources can be downloaded here : 
# ftp://ftp.unidata.ucar.edu/pub/netcdf/netcdf-4/
# examples : 
  wget -mnH --cut-dirs=1 https://curl.haxx.se/download/curl-7.67.0.tar.gz
  wget -mnH https://www.zlib.net/zlib-1.2.11.tar.gz
# DO NOT USE HDF5 VERSION BEFORE 1.8.14 -> memory leak
# DO NOT USE HDF5 VERSION AFTER 1.10.2 -> WRF model crash
  wget -mnH --cut-dirs=6 https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.10/hdf5-1.10.5/src/hdf5-1.10.5.tar.gz


# netcdf c and fortran sources can be downloaded here :
# https://www.unidata.ucar.edu/downloads/netcdf/index.jsp
# examples : 
  wget -mnH --cut-dirs=2 ftp://ftp.unidata.ucar.edu/pub/netcdf/netcdf-c-4.7.3.tar.gz
  wget -mnH --cut-dirs=2 ftp://ftp.unidata.ucar.edu/pub/netcdf/netcdf-fortran-4.5.2.tar.gz


# nco netCDF Operator can be downloaded here : 
# http://nco.sourceforge.net
# https://github.com/nco/nco/archive/
  wget -mnH --cut-dirs=1 http://dust.ess.uci.edu/nco/antlr-2.7.7.tar.gz
  wget -mnH --cut-dirs=2 ftp://ftp.gnu.org/gnu/gsl/gsl-2.5.tar.gz
  wget -mnH --cut-dirs=3 https://github.com/nco/nco/archive/4.8.1.tar.gz
  mv 4.8.1.tar.gz nco-4.8.1.tar.gz


# ncview can be downloaded here : 
# http://meteora.ucsd.edu/~pierce/ncview_home_page.html
  wget -mnH --cut-dirs=1  https://download.sourceforge.net/libpng/libpng-1.6.34.tar.gz
  wget -mnH --cut-dirs=2 ftp://cirrus.ucsd.edu/pub/ncview/ncview-2.1.8.tar.gz


# openssl can be downloaded here : 
  wget -mnH --cut-dirs=1 https://www.openssl.org/source/openssl-1.1.1d.tar.gz

# jpeg can be downloaded here :
  wget -mnH --cut-dirs=1 http://www.ijg.org/files/jpegsrc.v9d.tar.gz

# jasper can be downloaded here : 
  wget -mnH --cut-dirs=3 https://www.ece.uvic.ca/~frodo/jasper/software/jasper-2.0.14.tar.gz

# eccodes can be downloaded here : 
  wget -mnH --cut-dirs=3 https://confluence.ecmwf.int/download/attachments/45757960/eccodes-2.16.0-Source.tar.gz?api=v2

#  grib2 can be downloaded here : 
   wget -mnH --cut-dirs=3 https://www.nco.ncep.noaa.gov/pmb/codes/GRIB2/g2clib-1.6.0.tar

# cdo can be downloaded here : 
# https://code.mpimet.mpg.de/projects/cdo/files
  wget -mnH --cut-dirs=3 https://code.mpimet.mpg.de/attachments/download/18264/cdo-1.9.5.tar.gz

# ncl can be downloaded here : 
  wget -mnH --cut-dirs=3 https://www.earthsystemgrid.org/dataset/ncl.662.src/file/ncl_ncarg-6.6.2.tar.gz

fi

####################################################################################################
## Section 3 - download from ftp

if  [ ${ftp} -eq 1 ]; then

  # download all the tar.gz from the ifremer ftp 
  wget -mnH --cut-dirs=7 "ftp://ftp.ifremer.fr/ifremer/ww3/COURS/WAVE_DATA/NETCDF/TAR/*"

  # or copy all the tar.gz from the datarmor directory
  # rsync -a /home/datawork-WW3/COURS/WAVE_DATA/NETCDF/TAR2019/* .

fi

####################################################################################################
## Section 4 - extract tar to src

if [ ${extract} -eq 1 ]; then

  cd $NETCDF
 rm -rf src && mkdir -p src
 cd src

 echo "extract tar files"
# netcdf
 tar xf ../TAR/flex-2*.tar.gz
 tar xf ../TAR/libexpat-2*.tar.gz
 tar xf ../TAR/udunits-2*.tar.gz
 tar xf ../TAR/zlib-1.*.tar.gz
 tar xf ../TAR/hdf5-1.10.*.tar.gz
 tar xf ../TAR/netcdf-c-4*.tar.gz
 tar xf ../TAR/netcdf-fortran-*.tar.gz
 tar xf ../TAR/openssl-1.1.1d.tar.gz
 tar xf ../TAR/curl-7.*.tar.gz
# nco
 tar xf ../TAR/antlr-2*.tar.gz
 tar xf ../TAR/gsl-2*.tar.gz
 tar xf ../TAR/esmf_8_0*.tar.gz && mv esmf esmf-8.0.0
 tar xf ../TAR/nco-4.9*.tar.gz
# ncview
 tar xf ../TAR/libpng-1*.tar.gz
 tar xf ../TAR/ncview-2*.tar.gz
# cdo and ncl
 tar xf ../TAR/jpegsrc.v9d.tar.gz
 tar xf ../TAR/jasper-2.0.14.tar.gz
 tar xf ../TAR/eccodes-2.16.0-Source.tar.gz
 tar xf ../TAR/g2clib-1.6.0.tar
 tar xf ../TAR/cdo-1.9*.tar.gz
 tar xf ../TAR/ncl_ncarg-6.6.2.tar.gz

fi

####################################################################################################
## Section 5 - initialize compiler environment

# set environement
echo 'set env'
source /usr/share/Modules/3.2.10/init/bash
module purge

if [ "${compiler}" == "PGI" ]; then

  module load pgi/17.10
  module list

  export comp_c=pgcc
  export comp_cpp='pgcc -E'
  export comp_cxx='pgc++'
  export comp_f=pgf90

  export comp_mpic=mpicc
  export comp_mpicpp='mpicc -E'
  export comp_mpicxx=mpicxx
  export comp_mpif=mpif90

  export fcflags='-O3'
  export cflags='-O3 -fPIC'

  export ESMF_OS=Linux
  export ESMF_COMPILER=pgi
  export ESMF_COMM=mpiuni
  export ESMF_OPENMP=OFF



elif [ "${compiler}" == "GNU" ]; then

  module load impi/2018.1.163
  module list

  export comp_c=gcc
  export comp_cpp='gcc -E'
  export comp_cxx=g++
  export comp_f=gfortran

  export comp_mpic=mpigcc
  export comp_mpicpp='mpigcc -E'
  export comp_mpicxx=mpigxx
  export comp_mpif=mpif90

  export fcflags='-O3'
  export cflags='-O3 -fPIC'

  export ESMF_OS=Linux
  export ESMF_COMPILER=gfortran
  export ESMF_COMM=intelmpi
  export ESMF_OPENMP=OFF

elif [ "${compiler}" == "INTEL" ]; then

  module load intel-comp/18
  module load impi/2018.1.163
  module list

  export comp_c=icc
  export comp_cpp='icc -E'
  export comp_cxx=icpc
  export comp_f=ifort

  export comp_mpic=mpiicc
  export comp_mpicpp='mpiicc -E'
  export comp_mpicxx=mpiicpc
  export comp_mpif=mpiifort

  export fcflags='-O3'
  export cflags='-O3 -fPIC'

  export ESMF_OS=Linux
  export ESMF_COMPILER=intel
  export ESMF_COMM=intelmpi
  export ESMF_OPENMP=OFF

elif [ "${compiler}" == "MPT" ]; then

  module load intel-comp/18
  module load mpt/2.18
  module list


  export comp_c=icc
  export comp_cpp='icc -E'
  export comp_cxx=icpc
  export comp_f=ifort

  export comp_mpic='icc -lmpi'
  export comp_mpicpp='icc -lmpi -E'
  export comp_mpicxx='icpc -lmpi'
  export comp_mpif='ifort -lmpi'

  export fcflags='-O3'
  export cflags='-O3 -fPIC'

  export ESMF_OS=Linux
  export ESMF_COMPILER=intel
  export ESMF_COMM=mpt
  export ESMF_OPENMP=OFF
fi



####################################################################################################
## Section 6 - initialize environment variables

rm -rf ${NETCDF}/bin ${NETCDF}/sbin ${NETCDF}/include ${NETCDF}/lib
mkdir -p ${NETCDF}/include ${NETCDF}/lib ${NETCDF}/bin

echo 'compiler : ' $compiler
export comp_mpic_exe=$(echo $comp_mpic | awk -F' ' '{print $1}')
echo "$comp_mpic_exe"
export comp_mpif_exe=$(echo $comp_mpif | awk -F' ' '{print $1}')
echo "$comp_mpif_exe"
which $comp_mpic_exe
which $comp_mpif_exe

export OPT_CONF="CPPFLAGS=-I${NETCDF}/include LDFLAGS=-L${NETCDF}/lib --prefix=${NETCDF}"
echo "OPT_CONF : " $OPT_CONF

export PATH=${NETCDF}/bin::${PATH}
echo "PATH : " $PATH

export LD_LIBRARY_PATH=${NETCDF}/lib::${LD_LIBRARY_PATH}
echo "LD_LIBRARY_PATH : " $LD_LIBRARY_PATH

export CPATH=${NETCDF}/include::${CPATH}
echo "CPATH : " $CPATH



####################################################################################################
## Section 7 - netcdf libraries installation

cd $NETCDF/src

if [ ${netcdf} -eq 1 ]; then

# FLEX INSTALLATION
                cd flex-*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   --enable-static=no \
                   >& Configure-flex.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-flex.log
                echo "make install " $(basename ${PWD})
                make install >& Install-flex.log
                cd ..

# EXPAT INSTALLATION
                cd libexpat-*
                cd expat
                echo "configure " $(basename ${PWD})
                ./buildconf.sh >& Build_libexpat.log
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   --without-docbook \
                   --without-xmlwf  \
                   --enable-static=no \
                   >& Configure-libexpat.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-libexpat.log
                echo "make install " $(basename ${PWD})
                make install >& Install-libexpat.log
                cd ../..


# UDUNITS INSTALLATION
                cd udunits-2*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   --enable-static=no \
                   >& Configure-udunits.log
                echo "make " $(basename ${PWD})
                make -j 24  >& Compile-udunits.log
                echo "make install " $(basename ${PWD})
                make install >& Install-udunits.log
                cd ..

# ZLIB INSTALLATION
#                cd zlib-1*
#                echo "configure " $(basename ${PWD})
#                export CFLAGS="$cflags"
#                ./configure  \
#                   --prefix=${NETCDF}  \
#                   >& Configure-zlib.log
#                echo "make " $(basename ${PWD})
#                make -j 24   >& Compile-zlib.log
#                echo "make install " $(basename ${PWD})
#                make install >& Install-zlib.log
#                unset CFLAGS
#                cd ..

# HDF5 INSTALLATION
#                   --enable-debug=all \
		cd hdf5-*
                echo "configure " $(basename ${PWD})
                if [ "${compiler}" == "PGI" ]; then fpic='-fPIC'; else fpic=''; fi
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags $fpic" \
                   LIBS="-ludunits2" \
                   --enable-hl \
                   --enable-fortran \
                   --enable-parallel \
                   --enable-static=no \
                   >& Configure-hdf5.log
                echo "make " $(basename ${PWD})
                make -j 24   >& Compile-hdf5.log
                echo "make install " $(basename ${PWD})
                make install >& Install-hdf5.log
                cd ..

# NETCDF C INSTALLATION
#                   --enable-logging \
		cd netcdf-c-4*
                echo "configure " $(basename ${PWD})
                if [ "${compiler}" == "PGI" ]; then fpic='-fPIC'; else fpic=''; fi
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags $fpic" \
                   LIBS="-ludunits2" \
                   --enable-static=no \
                   --enable-parallel-tests \
                   --enable-shared \
                   >& Configure-netcdf-c.log
                echo "make " $(basename ${PWD})
		make -j 24   >& Compile-netcdf-c.log
                echo "make install " $(basename ${PWD})
                make install >& Install-netcdf-c.log
                cd ..

# NETCDF FORTRAN INSTALLATION
 #                  --enable-logging \
		cd netcdf-fortran-*
                echo "configure " $(basename ${PWD})
                if [ "${compiler}" == "PGI" ]; then fpic='-fPIC'; else fpic=''; fi
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags $fpic" \
		   LIBS="-lhdf5hl_fortran -lhdf5_hl -lhdf5_fortran -lhdf5 -ldl -lm -lz -ludunits2" \
                   --enable-static=no \
                   --enable-parallel-tests \
                   --enable-shared \
                   >& Configure-netcdf-fortran.log
                echo "make " $(basename ${PWD})
		make -j 24   >& Compile-netcdf-fortran.log
                echo "make install " $(basename ${PWD})
                make install >& Install-netcdf-fortran.log
                cd ..

# OPENSSL INSTALLATION (shared by default)
# produce crypto and ssl libraries
                cd openssl-1.1.*
                echo "configure " $(basename ${PWD})
                ./config --prefix=${NETCDF} \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   FCFLAGS="$fcflags" \
                   LIBS="$CLIBS" \
                   no-static-engine \
                   >& Configure-openssl.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-openssl.log
                echo "make install " $(basename ${PWD})
                make install >& Install-openssl.log
                cd ..


# CURL INSTALLATION
                cd curl-7*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   --enable-static=no \
                   >& Configure-curl.log
                echo "make " $(basename ${PWD})
                make -j 24   >& Compile-curl.log
                echo "make install " $(basename ${PWD})
                make install >& Install-curl.log
                cd ..
fi


####################################################################################################
## Section 8 - nco installation

if [ ${nco} -eq 1 ]; then

# UPDATE ENVIRONMENT
export PATH=${PATH}::${NETCDF}/bin
CLIBS=$(${NETCDF}/bin/nc-config --libs)
echo "CLIBS : " $CLIBS

# ANTLR INSTALLATION
# no effect : --enable-static=no \
                cd antlr-*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   LIBS="$CLIBS" \
                   --enable-shared \
                   >& Configure-antlr.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-antlr.log
                echo "make install " $(basename ${PWD})
                make install >& Install-antlr.log
                cd ..

# GSL INSTALLATION
                cd gsl-*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   LIBS="$CLIBS" \
                   --enable-shared \
                   --enable-static=no \
                   >& Configure-gsl.log
                # bug fix for Portland
                cat config.h|sed 's;#define HAVE_INLINE 1;#undef HAVE_INLINE;g' > config.h.new
                mv config.h.new config.h
                echo "make " $(basename ${PWD})
                make -j 24  >& Compile-gsl.log
                echo "make install " $(basename ${PWD})
                make install >& Install-gsl.log
                cd ..

# ESMF INSTALLATION
                cd esmf-*
                export ESMF_DIR=$PWD
                export ESMF_INSTALL_PREFIX=$NETCDF
                export ESMF_NETCDF_INCLUDE=$NETCDF/include
                export ESMF_NETCDF_LIBPATH=$NETCDF/lib
                export ESMF_NETCDF=$NETCDF/bin/nc-config
                echo "make " $(basename ${PWD})
                make -j 24  >& Compile-esmf.log
                echo "make install " $(basename ${PWD})
                make install >& Install-esmf.log
                cd ..


# FLEX NEEDED FOR UDUNITS
# EXPAT NEEDED FOR UDUNITS
# UDUNITS NEEDED FOR NCO
# ESMF NEEDED FOR NCO GRID MAPPING

# NCO INSTALLATION
                cd nco-*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   UDUNITS2_PATH="$NETCDF" \
                   GSL_ROOT="$NETCDF" \
                   ESMF_ROOT="$NETCDF" \
                   ESMF_LIB="$(find $NETCDF/lib/libO/ -mindepth 1 -type d)" \
                   ESMF_INC="$NETCDF/include" \
                   LIBS="$CLIBS" \
                   --enable-ncap2 \
                   --disable-openmp \
                   --enable-shared \
                   --enable-static=no \
                   >& Configure-nco.log
                echo "make " $(basename ${PWD})
                make -j 24  >& Compile-nco.log
                echo "make install " $(basename ${PWD})
                make install >& Install-nco.log
                cd ..
fi


####################################################################################################
## Section 9 - ncview installation

if [ ${ncview} -eq 1 ]; then

# PNG INSTALLATION
                cd libpng-*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   LIBS="$CLIBS" \
                   --enable-static=no \
                   --with-zlib-prefix="${NETCDF}" \
                   >& Configure-libpng.log
                echo "make " $(basename ${PWD})
                make -j 24  >& Compile-libpng.log
                echo "make install " $(basename ${PWD})
                make install >& Install-libpng.log
                cd ..



# NCVIEW INSTALLATION
# MUST BE DONE ON THE FRONT-END NODE FOR X-SERVER ENABLING
                cd ncview-*
                echo "configure " $(basename ${PWD})
                ./configure  $OPT_CONF \
                   FC="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   CFLAGS="$cflags" CXXFLAGS="$cflags" FCFLAGS="$fcflags" \
                   LIBS="$CLIBS -ludunits2 -lpng -lpng16" \
                   --x-libraries=/usr/lib64 --x-includes=/usr/include/X11 \
                   >& Configure-ncview.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-ncview.log
                echo "make install " $(basename ${PWD})
                make install >& Install-ncview.log
                cd ..
fi

####################################################################################################
## Section 10 - cdo installation

if [ ${cdo} -eq 1 ]; then

# JPEG INSTALLATION
                cd jpeg-9d
                echo "configure " $(basename ${PWD})
                ./configure  --prefix=${NETCDF} \
                   --enable-static=no \
                   >& Configure-jpeg.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-jpeg.log
                echo "make install " $(basename ${PWD})
                make install >& Install-jpeg.log
                cd ..

module load cmake # for jasper and eccodes

# JASPER INSTALLATION
                cd jasper-2.0*
                echo "configure " $(basename ${PWD})
                cmake -G "Unix Makefiles" -H$PWD -B$PWD/build \
                      -DJPEG_INCLUDE_DIR=${NETCDF}/include \
                      -DJPEG_LIBRARY=${NETCDF}/lib/libjpeg.so \
                      -DCMAKE_INSTALL_PREFIX=${NETCDF} \
                   >& Configure-jasper.log
                cd build
                make clean all >& Clean-jasper.log
                make install >& Install-jasper.log
                cd ../..
                ln -rfsn ${NETCDF}/lib64/*so* ${NETCDF}/lib/

#. /appli/anaconda/versions/4.7.12/etc/profile.d/conda.sh
#conda activate /appli/anaconda/versions/4.7.12
module load conda/4.7.12

# ECCODES INSTALLATION
                cd eccodes-2.16.0-Source
                echo "configure " $(basename ${PWD})
                mkdir build
                cd build
                cmake -DNetCDF_INCLUDE_DIRS=${NETCDF}/include/netcdf.inc \
                      -DNetCDF_LIBRARIES=${NETCDF}/lib/libnetcdf.so \
                      -DJPEG_INCLUDE_DIR=${NETCDF}/include \
                      -DJPEG_LIBRARY=${NETCDF}/lib/libjpeg.so \
                      -DCMAKE_INSTALL_PREFIX=${NETCDF} \
                      -DENABLE_NETCDF=ON \
                       ../ >& Configure-eccodes.log
                make >& Compile-eccodes.log
                make install >& Install-eccodes.log
                #pip install --install-option="--prefix=${NETCDF}" eccodes-python >& PipInstall-eccodes.log
                cd ../..

#conda deactivate # deactivate conda
module unload conda/4.7.12
module unload cmake # for jasper and eccodes

# CDO INSTALLATION
                cd cdo-1.9*
                echo "configure " $(basename ${PWD})
                if [ "${compiler}" == "MPT" ]; then CLIBS="$CLIBS -L$MPI_ROOT/lib -lmpi -lmpi++abi1002 -lsma"; fi
#                if [ "${compiler}" == "PGI" ]; then CLIBS="$CLIBS -L$MPI_ROOT/lib -lmpi -lmpi_cxx -lnuma -loshmem"; fi
                ./configure  $OPT_CONF \
                   CFLAGS="$cflags $cpp" CXXFLAGS="$cflags $cpp" FFLAGS="$fcflags" \
                   FC="$comp_mpif" \
                   F77="$comp_mpif" \
                   CC="$comp_mpic" \
                   CPP="$comp_mpicpp" \
                   CXX="$comp_mpicxx" \
                   LIBS="$CLIBS -L${NETCDF}/lib -ljpeg" \
                   --enable-static=no \
                   --with-netcdf=yes \
                   --with-hdf5=yes \
                   --with-udunits2=yes \
                   --with-curl=yes \
                   --with-eccodes=yes \
                   --with-threads=yes \
                   --disable-openmp \
                   >& Configure-cdo.log
                echo "make " $(basename ${PWD})
                make -j 24 >& Compile-cdo.log
                echo "make install " $(basename ${PWD})
                make install >& Install-cdo.log
                cd ..
fi

####################################################################################################
## Section 11 - cdl installation

if [ ${cdl} -eq 1 ]; then

	echo 'NCL must be installed interactively'

# G2CLIB INSTALLATION : must be done interactively
#                cd g2clib-1.6*
#                rm -f libg2c_v1.6.0.a
#                echo "make " $(basename ${PWD})
#                # sed -e "s/DEFS=-DUSE_JPEG2000 -DUSE_PNG/DEFS=-DUSE_PNG/g" makefile > makefile
#                # sed -e "s:??:INC=-I${NETCDF}/include" makefile > makefile
#                # sed -e "s/CC=gcc/CC=${comp_c}/g" makefile > makefile
#                make >& Compile-g2clib.log
#                 cp libg2c_v1.6.0.a ${NETCDF}/lib/
#                cd ..

# NCL INSTALLATION : must be done interactively
#                cd ncl_ncarg-6.6*
#                echo "configure " $(basename ${PWD})
#                ./Configure done interactive
#                 export NCARG_ROOT=${NETCDF}
#                 make Everything >& Compile-ncl.log 
#                 make All >& Install-ncl.log


fi

####################################################################################################
## Section 12 - successfull exit message

echo ''
echo 'INSTALLATION CORRECTLY DONE'
echo ''
echo 'in order to use this netcdf library'
echo 'you must add this to your bashrc ou cshrc : '
echo ''
echo -n '   export PATH=${PATH}'
echo ":${NETCDF}/bin"
echo -n '   export CPATH=${CPATH}'
echo ":${NETCDF}/include"
echo -n '   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}'
echo ":${NETCDF}/lib"
echo 'or'
echo -n '   setenv PATH ${PATH}'
echo ":${NETCDF}/bin"
echo -n '   setenv CPATH ${CPATH}'
echo ":${NETCDF}/include"
echo -n '   setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}'
echo ":${NETCDF}/lib"

